package com.aizuda.boot.modules.flw.controller;

import com.aizuda.boot.modules.flw.FlowHelper;
import com.aizuda.boot.modules.flw.entity.dto.FlwProcessDTO;
import com.aizuda.boot.modules.flw.entity.dto.ProcessStartDTO;
import com.aizuda.boot.modules.flw.entity.vo.FlwProcessCategoryVO;
import com.aizuda.boot.modules.flw.service.IFlwProcessService;
import com.aizuda.core.api.ApiController;
import com.baomidou.kisso.annotation.Permission;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 流程定义 前端控制器
 *
 * @author 青苗
 * @since 2023-09-07
 */
@Tag(name = "流程定义")
@RestController
@AllArgsConstructor
@RequestMapping("/v1/process")
public class ProcessController extends ApiController {
    private IFlwProcessService flwProcessService;

    @Operation(summary = "获取所有分类流程定义列表")
    @Permission("flw:process:listCategory")
    @Parameters({
            @Parameter(name = "keyword", description = "关键词")
    })
    @PostMapping("/list-category")
    public List<FlwProcessCategoryVO> listCategory(@RequestParam(required = false) String keyword) {
        return flwProcessService.listCategoryAll(keyword);
    }

    @Operation(summary = "获取发起分类流程定义列表")
    @Permission("flw:process:listStart")
    @Parameters({
            @Parameter(name = "keyword", description = "关键词")
    })
    @PostMapping("/list-launch")
    public List<FlwProcessCategoryVO> listLaunch(@RequestParam(required = false) String keyword) {
        return flwProcessService.listLaunch(keyword);
    }

    @Operation(summary = "发起流程")
    @Permission("flw:process:launch")
    @PostMapping("/launch")
    public boolean launchProcess(@RequestBody ProcessStartDTO dto) {
        return flwProcessService.launchProcess(dto, FlowHelper.getFlowCreator());
    }

    @Operation(summary = "根据 id 获取节点 map 列表")
    @Parameters({
            @Parameter(name = "id", description = "流程ID")
    })
    @Permission("sys:process:get")
    @PostMapping("/list-node-map")
    public List<Map<String, Object>> listNodeMap(@RequestParam Long id) {
        return flwProcessService.listNodeMapById(id);
    }

    @Operation(summary = "查询 id 信息")
    @Parameters({
            @Parameter(name = "id", description = "流程ID")
    })
    @Permission("sys:process:get")
    @GetMapping("/get")
    public FlwProcessDTO get(@RequestParam Long id) {
        return flwProcessService.getDtoById(id);
    }

    @Operation(summary = "查询 id 克隆流程定义信息")
    @Parameters({
            @Parameter(name = "id", description = "流程ID")
    })
    @Permission("sys:process:clone")
    @GetMapping("/clone")
    public boolean clone(@RequestParam Long id) {
        return flwProcessService.cloneById(id);
    }

    @Operation(summary = "创建添加")
    @Permission("flw:process:create")
    @PostMapping("/create")
    public Long create(@RequestBody FlwProcessDTO dto) {
        return flwProcessService.saveDto(dto);
    }

    @Operation(summary = "发布流程")
    @Parameters({
            @Parameter(name = "id", description = "流程ID")
    })
    @Permission("flw:process:create")
    @PostMapping("/release")
    public boolean release(@RequestParam Long id) {
        return flwProcessService.releaseById(id);
    }

    @Operation(summary = "根据流程定义ID删除流程定义相关信息")
    @Parameters({
            @Parameter(name = "id", description = "主键ID")
    })
    @Permission("flw:process:delete")
    @PostMapping("/delete")
    public boolean delete(@RequestParam Long id) {
        return flwProcessService.removeProcessInfo(id);
    }

    @Operation(summary = "根据流程定义ID更新排序")
    @Permission("flw:process:updateSort")
    @Parameters({
            @Parameter(name = "id", description = "主键ID"),
            @Parameter(name = "sort", description = "排序号")
    })
    @PostMapping("/update-sort-{id}")
    public boolean updateSort(@PathVariable("id") Long id, @RequestParam Integer sort) {
        return flwProcessService.updateSortById(id, sort);
    }

    @Operation(summary = "根据流程定义ID更新流程状态")
    @Permission("flw:process:updateSate")
    @Parameters({
            @Parameter(name = "id", description = "主键ID"),
            @Parameter(name = "state", description = "流程状态 0，不可用 1，可用")
    })
    @PostMapping("/update-state-{id}")
    public boolean updateSate(@PathVariable("id") Long id, @RequestParam Integer state) {
        return flwProcessService.updateSateById(id, state);
    }

}
