package com.aizuda.boot.modules.flw.service.impl;

import com.aizuda.boot.modules.flw.entity.FlwProcessActor;
import com.aizuda.boot.modules.flw.entity.FlwProcessCategory;
import com.aizuda.boot.modules.flw.entity.FlwProcessConfigure;
import com.aizuda.boot.modules.flw.entity.FlwProcessPermission;
import com.aizuda.boot.modules.flw.entity.dto.FlwProcessDTO;
import com.aizuda.boot.modules.flw.entity.dto.FlwProcessPermissionDTO;
import com.aizuda.boot.modules.flw.entity.dto.ProcessStartDTO;
import com.aizuda.boot.modules.flw.entity.vo.FlwProcessCategoryVO;
import com.aizuda.boot.modules.flw.entity.vo.FlwProcessVO;
import com.aizuda.boot.modules.flw.mapper.FlowlongMapper;
import com.aizuda.boot.modules.flw.service.*;
import com.aizuda.core.api.ApiAssert;
import com.aizuda.service.web.UserSession;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.flowlong.bpm.engine.FlowLongEngine;
import com.flowlong.bpm.engine.core.FlowCreator;
import com.flowlong.bpm.engine.entity.FlwInstance;
import com.flowlong.bpm.engine.entity.FlwProcess;
import com.flowlong.bpm.engine.model.ModelHelper;
import com.flowlong.bpm.engine.model.NodeAssignee;
import com.flowlong.bpm.engine.model.NodeModel;
import com.flowlong.bpm.engine.model.ProcessModel;
import com.flowlong.bpm.mybatisplus.mapper.FlwProcessMapper;
import lombok.AllArgsConstructor;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 流程分类 服务实现类
 *
 * @author 青苗
 * @since 2023-09-07
 */
@Service
@AllArgsConstructor
public class FlwProcessServiceImpl extends ServiceImpl<FlwProcessMapper, FlwProcess> implements IFlwProcessService {
    private IFlwProcessCategoryService flwProcessCategoryService;
    private IFlwProcessPermissionService flwProcessPermissionService;
    private IFlwProcessConfigureService flwProcessConfigureService;
    private IFlwProcessActorService flwProcessActorService;
    private IFlwProcessFormService flwProcessFormService;
    private FlowlongMapper flowlongMapper;
    private FlowLongEngine flowLongEngine;

    @Override
    public List<FlwProcessCategoryVO> listCategoryAll(String keyword) {
        return this.listCategoryVO(keyword, false);
    }

    public List<FlwProcessCategoryVO> listCategoryVO(String keyword, boolean launch) {
        List<FlwProcessCategory> categoryList = flwProcessCategoryService.list();
        if (CollectionUtils.isEmpty(categoryList)) {
            return null;
        }
        // 全部查询
        List<FlwProcessCategoryVO> voList = new ArrayList<>();
        List<FlwProcessVO> flwProcessVOList;
        if (launch) {
            flwProcessVOList = flowlongMapper.selectLaunchProcessList();
        } else {
            flwProcessVOList = flowlongMapper.selectFlwProcessList();
        }
        boolean voIsNotEmpty = CollectionUtils.isNotEmpty(flwProcessVOList);
        if (null == keyword) {
            // 不存在关键词查询
            for (FlwProcessCategory fpc : categoryList) {
                if (voIsNotEmpty) {
                    List<FlwProcessVO> processList = flwProcessVOList.stream().filter(f -> Objects.equals(fpc.getId(), f.getCategoryId())).toList();
                    if (CollectionUtils.isNotEmpty(processList)) {
                        FlwProcessCategoryVO vo = FlwProcessCategoryVO.of(fpc);
                        vo.setProcessList(processList);
                        voList.add(vo);
                    }
                } else if (!launch) {
                    // 非发起列表情况
                    voList.add(FlwProcessCategoryVO.of(fpc));
                }
            }
        } else {
            // 关键词查询
            for (FlwProcessCategory fpc : categoryList) {
                if (voIsNotEmpty) {
                    List<FlwProcessVO> processList = flwProcessVOList.stream().filter(t -> Objects.equals(t.getCategoryId(), fpc.getId())
                            && t.getProcessName().contains(keyword)).toList();
                    if (fpc.getName().contains(keyword)) {
                        // 匹配分类名称
                        if (CollectionUtils.isEmpty(processList)) {
                            if (launch) {
                                continue;
                            }
                            FlwProcessCategoryVO vo = FlwProcessCategoryVO.of(fpc);
                            vo.setProcessList(flwProcessVOList.stream().filter(f -> Objects.equals(f.getCategoryId(), fpc.getId())).toList());
                            voList.add(vo);
                        } else {
                            FlwProcessCategoryVO vo = FlwProcessCategoryVO.of(fpc);
                            vo.setProcessList(processList);
                            voList.add(vo);
                        }
                    } else if (CollectionUtils.isNotEmpty(processList)) {
                        // 匹配流程定义
                        FlwProcessCategoryVO vo = FlwProcessCategoryVO.of(fpc);
                        vo.setProcessList(processList);
                        voList.add(vo);
                    }
                } else if (!launch) {
                    // 非发起列表情况
                    voList.add(FlwProcessCategoryVO.of(fpc));
                }
            }
        }
        return voList;
    }

    @Override
    public List<FlwProcessCategoryVO> listLaunch(String keyword) {
        return this.listCategoryVO(keyword, true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean launchProcess(ProcessStartDTO dto, FlowCreator flowCreator) {
        Optional<FlwInstance> opt = flowLongEngine.startInstanceById(dto.getProcessId(), flowCreator);
        return opt.filter(flwInstance -> flwProcessFormService.saveForm(flwInstance.getId(), dto.getProcessForm())).isPresent();
    }

    @Override
    public List<Map<String, Object>> listNodeMapById(Long id) {
        FlwProcess flwProcess = this.checkById(id);
        return ModelHelper.getNodeMapList(flwProcess.model().getNodeConfig(), null);
    }

    @Override
    public FlwProcessDTO getDtoById(Long id) {
        FlwProcess flwProcess = this.checkById(id);
        FlwProcessDTO dto = FlwProcessDTO.of(flwProcess);
        // 流程权限
        List<FlwProcessPermission> flwProcessPermissions = flwProcessPermissionService.getByProcessId(id);
        if (CollectionUtils.isNotEmpty(flwProcessPermissions)) {
            dto.setProcessPermissionList(flwProcessPermissions.stream().map(FlwProcessPermissionDTO::of).toList());
        }
        // 流程配置
        FlwProcessConfigure configure = flwProcessConfigureService.getByProcessId(id);
        if (null != configure) {
            dto.setCategoryId(configure.getCategoryId());
            dto.setProcessSetting(configure.getProcessSetting());
            dto.setProcessForm(configure.getProcessForm());
        }
        return dto;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long saveDto(FlwProcessDTO dto) {
        ApiAssert.fail(null == dto.getCategoryId(), "流程定义分类ID不存在");
        // 流程定义ID
        Long processId = dto.getProcessId();
        FlwProcess flwProcess = new FlwProcess();
        flwProcess.setProcessKey(dto.getProcessKey());
        flwProcess.setProcessName(dto.getProcessName());
        flwProcess.setProcessIcon(dto.getProcessIcon());
        flwProcess.setUseScope(dto.getUseScope());
        flwProcess.setModelContent(dto.getModelContent());
        flwProcess.setProcessState(0); // 创建 0 不可用状态，需要发布后才可以使用
        flwProcess.setRemark(dto.getRemark());
        if (null == processId) {
            // 设置创建者信息
            UserSession userSession = UserSession.getLoginInfo();
            flwProcess.setCreateId(String.valueOf(userSession.getId()));
            flwProcess.setCreateBy(userSession.getUsername());
            flwProcess.setCreateTime(new Date());
            ApiAssert.fail(!super.save(flwProcess), "流程定义保存失败");
            processId = flwProcess.getId();
        } else {
            flwProcess.setId(processId);
            ApiAssert.fail(!super.updateById(flwProcess), "流程定义保存失败");
        }
        // 流程定义权限
        List<FlwProcessPermissionDTO> processPermissionList = dto.getProcessPermissionList();
        if (CollectionUtils.isNotEmpty(processPermissionList)) {
            ApiAssert.fail(!flwProcessPermissionService.saveProcessPermissions(processId, processPermissionList),
                    "流程定义管理权限保存失败");
        }
        // 保存流程定义配置
        if (null != dto.getCategoryId()) {
            ApiAssert.fail(!flwProcessConfigureService.saveByDto(processId, dto), "流程定义配置保存失败");
        }
        return processId;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean removeProcessInfo(Long id) {
        // 检查流程定义操作权限
        this.checkOperateApproval(id);
        // 删除相关流程定义信息
        flwProcessActorService.removeByProcessId(id);
        flwProcessPermissionService.removeByProcessId(id);
        flwProcessConfigureService.removeByProcessId(id);
        // 级联删除相关信息
        flowLongEngine.processService().cascadeRemove(id);
        return true;
    }

    protected void checkOperateApproval(Long processId) {
        UserSession userSession = UserSession.getLoginInfo();
        if (null == userSession || !UserSession.isAdmin(userSession.getId())) {
            FlwProcessPermission fpp = getFlwProcessPermissionByProcessId(processId);
            ApiAssert.fail(null == fpp || !fpp.allowOperateApproval(), "无权限编辑操作审批流程");
        }
    }

    protected FlwProcessPermission getFlwProcessPermissionByProcessId(Long processId) {
        UserSession userSession = UserSession.getLoginInfo();
        return flwProcessPermissionService.getByUserIdAndProcessId(userSession.getId(), processId);
    }

    @Override
    public boolean updateSortById(Long id, Integer sort) {
        // 检查流程定义操作权限
        this.checkOperateApproval(id);
        // 更新流程定义排序
        FlwProcess flwProcess = new FlwProcess();
        flwProcess.setId(id);
        flwProcess.setSort(sort);
        return super.updateById(flwProcess);
    }

    @Override
    public boolean updateSateById(Long id, Integer state) {
        // 检查流程定义操作权限
        this.checkOperateApproval(id);
        // 更新流程定义排序
        FlwProcess flwProcess = new FlwProcess();
        flwProcess.setId(id);
        flwProcess.setProcessState(Objects.equals(1, state) ? 1 : 0);
        return super.updateById(flwProcess);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean cloneById(Long id) {
        FlwProcessDTO dto = this.getDtoById(id);
        dto.setProcessId(null);
        return null != this.saveDto(dto);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean releaseById(Long id) {
        FlwProcess flwProcess = super.getById(id);
        ApiAssert.isEmpty(flwProcess, "指定ID流程实例不存在");
        if (1 == flwProcess.getProcessState()) {
            // 可用状态直接返回结果
            return true;
        }

        // 读取流程发起权限
        ProcessModel processModel = flwProcess.model();
        NodeModel nodeModel = processModel.getNodeConfig();
        ApiAssert.isEmpty(nodeModel, "流程定义配置有误");
        List<NodeAssignee> nodeRoleList = nodeModel.getNodeRoleList();
        if (CollectionUtils.isNotEmpty(nodeRoleList)) {
            flwProcessActorService.saveProcessActors(id, nodeRoleList.stream().map(t -> {
                FlwProcessActor fpa = new FlwProcessActor();
                fpa.setProcessId(id);
                fpa.setActorType(0);
                fpa.setActorId(Long.valueOf(t.getId()));
                fpa.setActorName(t.getName());
                return fpa;
            }).toList());
        }

        // 设置流程为可用状态
        FlwProcess temp = new FlwProcess();
        temp.setId(id);
        temp.setProcessState(1);
        return super.updateById(temp);
    }
}
