package com.aizuda.boot.modules.flw.service.impl;

import com.aizuda.boot.modules.flw.entity.FlwProcessForm;
import com.aizuda.boot.modules.flw.entity.dto.*;
import com.aizuda.boot.modules.flw.entity.vo.*;
import com.aizuda.boot.modules.flw.mapper.FlowlongMapper;
import com.aizuda.boot.modules.flw.service.IFlwProcessApprovalService;
import com.aizuda.boot.modules.flw.service.IFlwProcessFormService;
import com.aizuda.boot.modules.flw.service.IProcessTaskService;
import com.aizuda.core.api.ApiAssert;
import com.aizuda.core.api.PageParam;
import com.aizuda.service.web.UserSession;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.flowlong.bpm.engine.FlowLongEngine;
import com.flowlong.bpm.engine.TaskService;
import com.flowlong.bpm.engine.core.FlowCreator;
import com.flowlong.bpm.engine.entity.FlwExtInstance;
import com.flowlong.bpm.engine.entity.FlwTask;
import com.flowlong.bpm.engine.entity.FlwTaskActor;
import com.flowlong.bpm.engine.model.NodeModel;
import com.flowlong.bpm.engine.model.ProcessModel;
import com.flowlong.bpm.mybatisplus.mapper.FlwExtInstanceMapper;
import com.flowlong.bpm.mybatisplus.mapper.FlwTaskMapper;
import jakarta.annotation.Resource;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 流程任务 服务实现类
 *
 * @author 青苗
 * @since 2023-12-11
 */
@Service
public class ProcessTaskServiceImpl implements IProcessTaskService {
    @Resource
    private FlowlongMapper flowlongMapper;
    @Resource
    private FlwTaskMapper flwTaskMapper;
    @Resource
    private FlwExtInstanceMapper extInstanceMapper;
    @Resource
    private FlowLongEngine flowLongEngine;
    @Resource
    private IFlwProcessApprovalService flwProcessApprovalService;
    @Resource
    private IFlwProcessFormService flwProcessFormService;

    @Override
    public Page<PendingClaimTaskVO> pagePendingClaim(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = this.getProcessTaskDTO(pageParam);
        return flowlongMapper.selectPagePendingClaim(pageParam.page(), dto);
    }

    @Override
    public Page<PendingApprovalTaskVO> pagePendingApproval(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = this.getProcessTaskDTO(pageParam);
        return flowlongMapper.selectPagePendingApproval(pageParam.page(), dto);
    }

    @Override
    public Page<ProcessTaskVO> pageMyApplication(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = this.getProcessTaskDTO(pageParam);
        return flowlongMapper.selectPageMyApplication(pageParam.page(), dto);
    }

    @Override
    public Page<ProcessTaskVO> pageMyReceived(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = this.getProcessTaskDTO(pageParam);
        return flowlongMapper.selectPageMyReceived(pageParam.page(), dto);
    }

    @Override
    public Page<ProcessTaskVO> pageApproved(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = this.getProcessTaskDTO(pageParam);
        return flowlongMapper.selectPageApproved(pageParam.page(), dto);
    }

    @Override
    public TaskApprovalVO approvalInfo(Long taskId) {
        FlwTask flwTask = flwTaskMapper.selectById(taskId);
        if (null != flwTask) {
            return null;
        }
        TaskApprovalVO vo = new TaskApprovalVO();
        vo.setTaskId(taskId);
        vo.setCreateId(flwTask.getCreateId());
        vo.setCreateBy(flwTask.getCreateBy());
        vo.setCreateTime(flwTask.getCreateTime());
        // 表单填写内容
        FlwProcessForm flwProcessForm = flwProcessFormService.getByInstanceId(flwTask.getInstanceId());
        if (null != flwProcessForm) {
            vo.setFormContent(flwProcessForm.getContent());
        }
        // 表单配置权限
        FlwExtInstance flwExtInstance = extInstanceMapper.selectById(flwTask.getInstanceId());
        ProcessModel processModel = flwExtInstance.model();
        NodeModel nodeModel = processModel.getNode(flwTask.getTaskName());
        Map<String, Object> extendConfig = nodeModel.getExtendConfig();
        if (null != extendConfig) {
            Object formConfig = extendConfig.get("formConfig");
            if (null != formConfig) {
                vo.setFormConfig((String) formConfig);
            }
        }
        vo.setProcessApprovals(flwProcessApprovalService.listByInstanceId(flwTask.getInstanceId()));
        return vo;
    }

    @Override
    public boolean viewed(Long taskId, FlowCreator flowCreator) {
        return flowLongEngine.taskService().viewTask(taskId, FlwTaskActor.ofFlowCreator(flowCreator));
    }

    /**
     * 获取流程任务DTO
     */
    private ProcessTaskDTO getProcessTaskDTO(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = pageParam.getData();
        if (null == dto) {
            dto = new ProcessTaskDTO();
        }
        UserSession userSession = UserSession.getLoginInfo();
        dto.setCreateId(userSession.getUserId());
        return dto;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean transfer(Long taskId, FlowCreator flowCreator, FlowCreator assigneeFlowCreator) {
        TaskService taskService = flowLongEngine.taskService();
        return taskService.transferTask(taskId, flowCreator, assigneeFlowCreator);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean delegate(Long taskId, FlowCreator flowCreator, FlowCreator assigneeFlowCreator) {
        TaskService taskService = flowLongEngine.taskService();
        return taskService.delegateTask(taskId, flowCreator, assigneeFlowCreator);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean reclaim(Long taskId, FlowCreator flowCreator) {
        TaskService taskService = flowLongEngine.taskService();
        return taskService.reclaimTask(taskId, flowCreator).isPresent();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean claim(Long taskId, FlowCreator flowCreator) {
        TaskService taskService = flowLongEngine.taskService();
        return null != taskService.claim(taskId, flowCreator);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean executeJump(Long taskId, String nodeName, FlowCreator flowCreator) {
        return flowLongEngine.executeJumpTask(taskId, nodeName, flowCreator);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean execute(FlowCreator flowCreator, ExecuteTaskDTO dto) {
        return flowLongEngine.executeTask(dto.getTaskId(), flowCreator, dto.getArgs());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean reject(FlowCreator flowCreator, RejectTaskDTO dto) {
        FlwTask flwTask = flowLongEngine.queryService().getTask(dto.getTaskId());
        ApiAssert.isEmpty(flwTask, "当前ID执行任务不存在");
        TaskService taskService = flowLongEngine.taskService();
        return taskService.rejectTask(flwTask, flowCreator, dto.getArgs()).isPresent();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean transfer(FlowCreator flowCreator, AssigneeTaskDTO dto) {
        TaskService taskService = flowLongEngine.taskService();
        return taskService.transferTask(dto.getTaskId(), flowCreator, dto.toFlowCreator());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean delegate(FlowCreator flowCreator, AssigneeTaskDTO dto) {
        TaskService taskService = flowLongEngine.taskService();
        return taskService.delegateTask(dto.getTaskId(), flowCreator, dto.toFlowCreator());
    }

    @Override
    public List<FlwHisTaskVO> listHisTaskByInstanceId(Long instanceId) {
        List<FlwHisTaskVO> voList = flowlongMapper.selectListHisTaskByInstanceId(instanceId);
        if (CollectionUtils.isNotEmpty(voList)) {
            List<FlwHisTaskActorVO> actorList = flowlongMapper.selectListHisTaskActorVOByInstanceId(instanceId);
            if (CollectionUtils.isNotEmpty(actorList)) {
                voList.forEach(t -> t.setActorList(actorList.stream().filter(v -> Objects.equals(v.getTaskId(), t.getId()))
                        .collect(Collectors.toList())));
            }
        }
        return voList;
    }

    @Override
    public boolean approvalComment(TaskApprovalDTO dto) {
        ApiAssert.isEmpty(dto.getContent(), "评论不能为空");

        return flwProcessApprovalService.comment(dto);
    }

    @Override
    public boolean approvalConsent(TaskApprovalDTO dto) {

        return flwProcessApprovalService.consent(dto);
    }

    @Override
    public boolean approvalRejection(TaskApprovalDTO dto) {

        return flwProcessApprovalService.rejection(dto);
    }
}
